<?php

// MIT License
// Copyright 2016 Thorsten Thormaehlen
// Contact: www.thormae.de

// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights 
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to
// do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR 
// A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
// CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
// OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

$userDataDir = "./polls/";

function cleanUpOldPolls($dir) {
  $secondsToLive = 60 * 60 * 24 * 8; // 8 days
  foreach (glob($dir . "poll*") as $pollDir) {
    if (is_dir($pollDir)) {
      if ((time() - filemtime($pollDir)) > $secondsToLive) {
        foreach (glob($pollDir . "/*") as $inDirFile) {
          unlink($inDirFile);
        }
        rmdir($pollDir);
        $dateStr = date("[Y/m/d G:i:s]");
        file_put_contents($dir . "clicker.log", $dateStr . "\t" . "Delete: Poll with name " . $pollDir . "\n", FILE_APPEND | LOCK_EX);
      }
    }
  }
}

// read post parameters
$poll = filter_input(INPUT_POST, 'pollNumber', FILTER_VALIDATE_INT);
$answers = filter_input(INPUT_POST, 'answersNumber', FILTER_VALIDATE_INT);
$requestType = filter_input(INPUT_POST, 'requestType', FILTER_VALIDATE_INT);

// check input
if (!$poll || !$answers || !$requestType) {
  die('Internal error: Wrong post format ' . ' poll=' . $poll . ' answers=' . $answers . " requestType=" . $requestType);
}

if (intval($requestType) < 1 || intval($requestType) > 4) {
  die('Internal error: requestType=' . $requestType);
}

$pollDir = "poll" . $poll;
$dateStr = date("[Y/m/d G:i:s]");

if ($requestType == "1") { // create poll
  
  cleanUpOldPolls($userDataDir, $dateStr);
  
  if (!file_exists($userDataDir)) {
    mkdir($userDataDir, 0777, true);
  }

  if (is_dir($userDataDir . $pollDir)) {
    die('A poll with number ' . $poll . ' already exists. Press &quot;Open&quot; or choose a different poll number.');
  } else {
    mkdir($userDataDir . $pollDir, 0777, true);
    file_put_contents($userDataDir . $pollDir . "/poll_info.txt", $answers, LOCK_EX);
    file_put_contents($userDataDir . "clicker.log", $dateStr . "\t" . "Create: Poll with name " . $pollDir . " and " . $answers . " answer options created \n", FILE_APPEND | LOCK_EX);
    echo "Success";
  }
}

if ($requestType == "2") { // get poll result
  $dir = $userDataDir . $pollDir;
  if (!is_dir($dir)) {
    die('A poll with number ' . $poll . ' does not exist. Polls are automatically deleted eight days after their creation.');
  } else {

    $infoFile = $dir . "/poll_info.txt";
    if (!file_exists($infoFile)) {
      die('Internal error: can not open ' . $infoFile);
    }
    $countOptions = intval(file_get_contents($infoFile));

    $counters = range(0, $countOptions - 1);
    for ($i = 0; $i < $countOptions; $i++) {
      $counters[$i] = 0;
    }

    $total = 0;
    $dh = opendir($dir);
    while (($file = readdir($dh)) !== false) {
      $f = $dir . "/" . $file;
      if (is_file($f)) {
        //echo "filename: " . $file . "\n";
        $fileParts = explode("_", $file);
        if (count($fileParts) > 2) {
          if ($fileParts[0] == "vote") {
            $ii = intval($fileParts[1]);
            if ($ii > 0 && $ii <= $countOptions) {
              $counters[$ii - 1] ++;
              $total++;
            }
          }
        }
      }
      //print_r( array_values( $counters ));
      //echo "\n";
    }
  }
  closedir($dh);
  echo "Success:" . $total;
  for ($i = 0; $i < $countOptions; $i++) {
    echo " " . $counters[$i];
  }
}

if ($requestType == "3") { // get number of answer options
  if (!is_dir($userDataDir . $pollDir)) {
    die('A poll with number ' . $poll . ' does not exist. Press the &quot;Admin&quot; button if you like to create a new poll with this number.');
  } else {
    $infoFile = $userDataDir . $pollDir . "/poll_info.txt";
    if (!file_exists($infoFile)) {
      die('Internal error: can not open ' . $infoFile);
    }
    echo "Success:" . file_get_contents($infoFile);
  }
}

if ($requestType == "4") { // vote
  if (!is_dir($userDataDir . $pollDir)) {
    die('A poll with number ' . $poll . ' does not exist.');
  } else {
    $randomFileName = "";
    do {
      $randomFileName = "vote_" . $answers . "_" . sprintf("%08d", rand() % 100000000) . ".txt";
    } while (file_exists($userDataDir . $pollDir . "/" . $randomFileName));

    touch($userDataDir . $pollDir . "/" . $randomFileName);
    echo "Success:" . $answers;
  }
}
?>